/*
Biblietoile - bibliothque de routine pour calcul de structure stellaire

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.util.*;

/** Bibliothque de routines pour le calcul d'une structure stellaire. Utilises notamment par les
    classes Etoile.java et StellarApplet.java.
*/
public class Biblietoile {

/*+-------------------------------------+
  | Dclaration des Constantes globales |
  +-------------------------------------+*/

  static final double C = 2.998e10;             // vitesse de la lumire en cm/s
  static final double G = 6.67e-8;              // constante de la gravitation en cm3/g/s2
  static final double SIGMA = 5.67e-5;          // constante de Bolzmann en erg/cm2/s/K4

  static final double ASTRO_UNIT = 1.4e13;      // unit astronomique en cm

  static final double SOLEIL_MASSE = 1.99e33;      // masse solaire en g
  static final double SOLEIL_LUM  = 3.83e33;       // luminosit solaire en erg/sec
  static final double SOLEIL_RAYON = 6.96e10;      // rayon du soleil en cm
  static final double SOLEIL_AGE = 4.6e9;          // age du soleil en annes
  static final double SOLEIL_T_CENTRAL = 15.5e6;   // Temp. centrale du soleil en K
  static final double SOLEIL_T_SURFACE = 5800;     // Temp. de surface du soleil en K
  static final double SOLEIL_D_CENTRAL = 147.2;    // densit centrale du soleil en g/cm3
  static final double SOLEIL_X = 0.71;             // X du soleil
  static final double SOLEIL_Y = 0.27;             // Y du soleil
  static final double SOLEIL_Z = 0.02;             // Z du soleil

  static final double FRACTION_D_AZOTE = 0.3;  // fraction d'azote dans les lments lourds
  static final double ENERGIE_H_EN_HE = 7e18;    // production d'nergie en convert. 1g d'H en He

/*+------------------------------------+
  | Dclaration des variables globales |
  +------------------------------------+*/

  double fitPrecision;   // precision dsire au point de raccordement
  double modelPrecision; // precision dsire pour le modle (=10*precision)
  int modIteration;      // compteur d'itrations
  int plotControl;       // pour sortir les profils T, D, R, L : 0=pas de transfert des valeurs,1=oui

  boolean plotMasseOuRayon; // Tracs en fonction de : false -> masse, true -> rayon

  double pi;             // 3.14159...
  double twoTo14;        // 2^.25
  int modeleCalcule;     // gal  0 suf si les valeurs calcules sont les valeurs courantes


  double x;    // fraction de masse d'hydrogne 
  double y;    // fraction de masse d'hlium
  double z;    // fraction de masse des "mtaux"
  double zAzote;    // fraction de masse d'azote
  double density;      // Densit (g/cm3)
  double temperature;  // Temprature (K)

  double mass;         // Masse de l'toile
  double radius;       // Rayon  la surface
  double luminosite;   // Luminosit  la surface
  double tSurface;     // Temprature de surface

  double tEffective;   // temprature effective de l'toile
  double tCentral;     // temprature centraler
  double dCentral;     // densit centrale
  double tZero;        // "vraie" temprature au centre
  double dZero;        // "vraie" densit au centre
  double dPcorr;       // correction de pression
  double gradlTlP;     // =dlnTdlnP mais en global
  double gradlDlP;     // =dlnDdlnP mais en global
  double rayonCoeur;   // pour rcuprer la valuer calcule dans dehors pour extrapolation AR
  double fitMassAuto;  // masse au point de jonction calcul automatiquement
  double minDeltaGlob; // le minimum maximorum pour recherche auto. du point de jonction


  double fitMass;         // masse au point de jonction
  double fitMassFraction; // fraction de masse de l'toile au point de jonction
  double densiteSoleil;      // densit solaire moyenne
  double constGrrad;      // constante sun_lum/sun_mass/(16*pi*c*G) // approx. 2.e-5 cgs
  double constdPdM;       // constante G/4/pi*(Sun_mass/Sun_radius^2)^2


  double kOpacite;      // Opacit (cm^2/g)
  double kElectron;     // Opacit due  la diffusion par les lectrons
  double kKramer;       // Opacit de Kramers
  double kHMoins;       // Opacit de l'ion H-
  double kMoleculaire;  // Opacit molculaire
  double kRadiative;    // Opacit radiative totale
  double kConductive;   // Opacit due  la conductivit lectronique

  double puissancepp;       // puissance engendre par p-p
  double puissanceCNO;      // puissance engendre par CNO
  double puissance3alpha;   // puissance engendre par 3 alpha
  double puissance;         // puissance engendre par p-p + CNO
  double dxdt;          // taux de dpltion de l'hydrogne
  double dydt;          // taux de dpltion de l'hlium

  double pression;      // pression totale, rayonnement+ions+lectrons (dyne/cm^2)
  double dLogPdLogT;    // d log(p)/d log(T)  densit constante
  double dLogPdLogD;    // d log(p)/d log(density)  temprature constante
  double pressionRad;   // pression de rayonnement
  double pressionIon;   // pression due aux ions
  double pressionElect; // pression lectronique
  double pressionGaz;   // pression du gaz, ions+lectrons
  double pressionDegR;  // pression des lectrons dgnrs relativistes 
  double pressionDegNR; // pression des lectrons dgnrs non-relativistes 
  double pressionDeg;   // pression des lectrons dgnrs
  double pressionNon;   // pression des lectrons non-dgnrs, loi des gaz parfaits
  double dLnTdLnPrad;   // gradient radiatif de temprature 
  double dLnTdLnPad;    // d logT/d logP  entropie constante ; gradient de T adiabatique

  double maxParamStep[] = new double[5];  // Incrment/dcrment max. pour chacun des paramtres
  double corrections[] = new double[5];   // corrections calcules par correctionValeur
  double maxCorrect[] = new double[5];    // corrections max. effectues par correctionValeur

/*+------------------------------------+
  | Dclaration des sorties graphiques |
  +------------------------------------+*/

  static final int keepSize = 50;         // dimension max des tableaux graphiques

  double M_array[] = new double[keepSize];  // profil de masse
  double T_array[] = new double[keepSize];  // profil de temprature
  double D_array[] = new double[keepSize];  // profil de densit
  double R_array[] = new double[keepSize];  // profil du rayon
  double L_array[] = new double[keepSize];  // profil de luminosit
  double K_array[] = new double[keepSize];  // profil d'opacit 
  double E_array[] = new double[keepSize];  // profil de taux de production d'nergie
  double P_array[] = new double[keepSize];  // profil de pression
  
  Vector vecteurXin = new Vector();  // vecteur pour stocker les abscisses des profils produits par dedans
  Vector vecteurTin = new Vector();  // profil de temprature
  Vector vecteurDin = new Vector();  // profil de densit
  Vector vecteurRin = new Vector();  // rayon ou profil de masse
  Vector vecteurLin = new Vector();  // profil de luminosit
  Vector vecteurXout = new Vector(); // comme ci-dessus mais pour dehors
  Vector vecteurTout = new Vector();
  Vector vecteurDout = new Vector();
  Vector vecteurRout = new Vector();
  Vector vecteurLout = new Vector();
  
/** Mthode pour convertir les nombres rels en chane de caractres avec choix du format d'affichage.
    On entre le nombre, le nombre de dcimales souhaites, et un boolen  true pour avoir un exposant.
*/
  public String arrondi(double nombre, int decimales, boolean scienti) {
      
    double exposant = 0, mantisse = 0;
    String nombreArrondi = null;
    boolean signe = false;
    
    if ( nombre != 0 ) {
      if ( nombre < 0 ) {
        nombre = -nombre;
        signe = true;
      }
      
      if ( !scienti ) {
        nombre = nombre * Math.pow(10.,(double)decimales);
        nombre = (double)Math.round(nombre);
        nombre = nombre / Math.pow(10.,(double)decimales);
    
        nombreArrondi = Double.toString(nombre);
      }
      
      if ( scienti | (Math.log(nombre)/Math.log(10.)) >= 6. ) {
        exposant = Math.floor(Math.log(nombre)/Math.log(10.));
        mantisse = nombre/Math.pow(10.,exposant);
        nombreArrondi = arrondi(mantisse,decimales,false)+"E"+Integer.toString((int)exposant);
      }
    
      if ( signe ) {
        nombreArrondi = "-"+nombreArrondi;
      }
    }
    else {
      nombreArrondi = "0.";
    }

    return nombreArrondi;
  }

/** Pour initialiser le calcul : composition chimique, quelques constantes utiles.
*/
  public void initialiseModele(){

    // Variable(s) locale(s)
    // ---------------------
    int i = 0;
    double intermediaire = 0.; // variable de type double pour tests

    pi = 4.0*Math.atan(1);
    twoTo14 = Math.pow(2.,0.25); // 2^.25

    x = SOLEIL_X; //  fraction de masse d'hydrogne
    y = SOLEIL_Y; //  fraction de masse d'hlium
    z = SOLEIL_Z; //  fraction de masse des "mtaux"
    zAzote = z*FRACTION_D_AZOTE; //  fraction de masse d'azote

    density = SOLEIL_D_CENTRAL;     // densit
    temperature = SOLEIL_T_CENTRAL; // temprature
    radius = Math.sqrt(luminosite)/Math.pow((tEffective/SOLEIL_T_SURFACE),2);

    constGrrad  = (SOLEIL_LUM/SOLEIL_MASSE)/(16.0*pi*C*G);       // utilis dans EvaluateEq
    constdPdM   = G/4.0/pi*Math.pow((SOLEIL_MASSE/Math.pow(SOLEIL_RAYON,2)),2); // utilis dans EvaluateEq
    densiteSoleil  = SOLEIL_MASSE/(4.0*pi*SOLEIL_RAYON*SOLEIL_RAYON*SOLEIL_RAYON/3.0); // densit moyenne du Soleil

    maxParamStep[0] = 0.20; // Pas maximal pour la temprature - au lieu de .05
    maxParamStep[1] = 0.20; // Pas maximal pour la densit - au lieu de .15
    maxParamStep[2] = 0.20; // Pas maximal pour le rayon - au lieu de .05
    maxParamStep[3] = 0.20; // Pas maximal pour la luminosit
    maxParamStep[4] = 0.20; // Pas maximal pour la masse

    fitPrecision    = 0.001; // prcision souhaite au point de jonction
    modelPrecision  = 10.*fitPrecision; // prcision souhaite pour le modle

// corrections max. pour chaque paramtre dans correctionValeur
    maxCorrect[0] = 0.03; // Teff
    maxCorrect[1] = 0.10; // Lum
    maxCorrect[2] = 0.02; // Tcen - au lieu de .01
    maxCorrect[3] = 0.10; // Dcen
    maxCorrect[4] = 0.00; // Masse

    // nettoyage des tableaux
    double M_array  = 0.;
    double T_array  = 0.;
    double D_array  = 0.;
    double R_array  = 0.;
    double L_array  = 0.;
    double K_array  = 0.;
    double E_array  = 0.;
    double P_array  = 0.;

  }

/*+--------------------------------------------+
  | Mthodes de calcul des processus physiques |
  +--------------------------------------------+*/

/** Calcule la production d'nergie par la chane p-p et le cycle CNO. 
    Expression de KW. Accord avec BP sauf pour les facteurs d'crantage 
    d'ordre lev.<br>
    Entre : x,y,z,zAzote,densite,temperature<br>
    Sortie : puissancepp, puissanceCNO, puissance (erg/g/sec), 
             dxdt (g/g/sec) (taux de disparition de H)
*/
  public void energieNucleaire(double xx,double yy,double zz,double zn,double d,double t){

    // Variable(s) locale(s)
    // ---------------------
    double t6;        // temp./10^6
    double t6to13;    // t6  la puissance 1/3
    double t6to23;    // t6  la puissance 2/3 
    double screenpp;  // facteur d'crantage pour pp
    double ecranCNO; // facteur d'crantage pour cno
    double tempo;

    puissancepp  = 1.0e-30;   // pas de production d'nergie  basse T
    puissanceCNO = 1.0e-30;
    puissance    = 1.0e-30;

    if (t > 1.0e6) {
       t6       = t/1.0e6;                 // T6
       t6to13   = Math.pow(t6,0.3333);     // T6  la puissance 1/3 
       t6to23   = t6to13*t6to13;           // T6  la puissance 2/3 
       tempo= (64.33-152.28/t6to13);   // au lieu de 64.24, 152.313
       if (tempo > -30.0) {
          ecranCNO = 1+0.0027*t6to13-0.00778*t6to23-0.000149*t6;
          if (ecranCNO < 1.0) {
            ecranCNO = 1.0;
          }
          puissanceCNO = xx*zn*ecranCNO*d*Math.exp(tempo)/t6to23;
       }
       screenpp = 1+0.0123*t6to13+0.0109*t6to23+0.0009*t6;
       puissancepp  = xx*xx*screenpp*d*Math.exp(14.68-33.80/t6to13)/t6to23; // au lieu de 14.54,33.81
       puissance    = puissancepp+puissanceCNO;
    }

    dxdt = -1.667e-19*puissance; // taux de disparition de l'hydrogne

  }

/** Production d'nergie par la raction 3 alpha.
    Expression de KW. Accord avec BP sauf pour les facteurs d'crantage 
    d'ordre lev.<br>
    Entre : x,y,z,zAzote,densite,temperature<br>
    Sortie : puissance3alpha (erg/g/sec), 
             dydt (g/g/sec) (taux de disparition de He)
*/
  public void energieNucleaireAlpha(double xx,double yy,double zz,double zn,double d,double t){
  
    // Variable(s) locale(s)
    // ---------------------
    double t8;

    puissance3alpha = 1.0e-30;

    if(t > 1.0e8) {
       t8 = t/1.0e8;
       puissance3alpha = 5.09e11*(d*d)*Math.pow((yy/t8),3)*Math.exp(-44.027/t8);
    }

    dydt = -1.7e-18*puissance3alpha;    // taux de disparition de l'hlium

  }

/** Calcul des opacits.<br>
    Entre : x,y,z,densite,temperature<br>
    Sortie : opacit (cm^2/g) et ses diffrentes composantes
*/
  public void opacite(double xx,double yy,double zz,double d,double t){

    // Variable(s) locale(s)
    // ---------------------
    double zAverage;

    // Constantes(s) locale(s)
    // -----------------------
    final double AA = 2;     // d'aprs BP au lieu de 6.0 selon Kramers
    final double BB = 1;     // d'aprs BP au lieu de .001 H-
    final double CC = 1;     // d'aprs BP au lieu de .001 molecular

    zAverage = xx+2*yy+8*zz;    // au lieu de 4*yy ?

/* Calcul de l'opacit due  la diffusion lectronique  haute T avec correction
de densit - moyennement prcise (terme dominant selon KW, correction selon
BP d'aprs AC)
*/
    kElectron = 0.2*(1+xx)/((1+2.7e11*d/t/t)*(1+Math.pow((t/4.5e8),0.86)));

/* Opacit de Kramer : dcrit approximativement l'opacit des transitions 
libre-libre, li-libre, li-li.
*/
    //   kKramer = (13.7/AA)*(1+xx)*(0.0010+zz)*d*Math.pow((1.0e7/t),3.5) // formule de BP  

    kKramer = (4.56/AA)*(1+xx)*(0.0025+zz)*d*Math.pow((1.0e7/t),3.5);     // formule de Stein 

/* Opacits approximatives de H- et molculaire 
*/
    if(t > 4.0e4) {
       kHMoins = 65.0*Math.pow((zz*d),2)*Math.pow((4.0e4/3000.0),7.7);  // valeur constante
       kMoleculaire = 1.0e-5;
    }
    else {
       kHMoins = 65.0*Math.pow((zz*d),2)*Math.pow((t/3000.0),7.7); // BP multiple ceci par BB
       kMoleculaire = 0.1*zz;                                        // BP multiple ceci par CC
    }

    // Opacit totale radiative

    kRadiative = kMoleculaire + 1./(1./kHMoins+1./(kElectron+kKramer));

    // Opacit par conductivit lectronique, approximative

    if(d < 1.0e-5) {
       kConductive = 1.0e10;
    }
    else {
       kConductive = zAverage*(2.6e-7)*Math.pow((t/d),2)*(1+Math.pow((d/2.0e6),0.66667));
    }

    // Opacit totale

    kOpacite = 1./(1./kRadiative+1./kConductive);

  }

/** Calcul de l'quation d'tat et des grandeurs thermodynamiques. Expressions selon BP.<br>
    Entre : x,y,z,densit,temprature<br>
    Sortie : pression, pressionGaz, pressionRad (dynes/cm^2), 
             dLogPdLogT, dLogPdLogD, dLnTdLnPad <br>
    L'ionisation totale est suppose, les lectrons peuvent tre partiellement
    dgnrs et partiellement relativistes
*/
  public void equationDEtat(double xx,double yy,double zz,double d,double t){

    // Constantes(s) locale(s)
    // -----------------------
    final double radconst = 2.523e-15;      // rad const a/3 (erg/CC/K^4)
    final double ionconst = 0.8263e8;       // const de Boltzmann/masse du proton (erg/g/K)
    final double const1   = 1.0036e13;      // KW -pour gaz d'e- dgnr non-relativiste
    final double const2   = 1.2435e15;      // KW -pour gaz d'e- dgnr relativiste

    // Variable(s) locale(s)
    // ---------------------
    double poidsMolMoyenIon;   // Poids molculaire moyen des ions; on prend z de l'oxygne
    double poidsMolMoyenElec;  // Poids molculaire moyen par lectron libre, ionisation totale
    double relativiste;    // A quel point les e- sont-ils relativistes ?
    double dLogPEdLogT;     // d log(P elect)/d log(T)  densit constante
    double dLogPEdLogD;     // d log(P elect)/d log(densit)  T constante
    double qt;              // T*Cv, o Cv est la chaleur spcifique  V constant (par gramme)
    double qr;              // qt*(d log(T)/d log(densit))  entropie constante

    poidsMolMoyenIon = 1.0/(xx+yy/4.0+zz/16.0);      // Poids molculaire moyen des ions
    poidsMolMoyenElec  = 2.0/(1.0+xx);               // Poids molculaire moyen par lectron libre

    pressionRad   = radconst*(Math.pow(t,4));    // pression du rayonnement
    pressionIon   = d*t*ionconst/poidsMolMoyenIon;  // pression des ions
    pressionDegR  = const2*Math.pow((d/poidsMolMoyenElec),(4.0/3.0));
    pressionDegNR = const1*Math.pow((d/poidsMolMoyenElec),(5.0/3.0));
    pressionDeg   = pressionDegNR/Math.pow((1.0+Math.pow((pressionDegNR/pressionDegR),2)),2);
    pressionNon   = d*t*ionconst/poidsMolMoyenElec;
    pressionElect = pressionNon*Math.pow((1.0+Math.pow((pressionDeg/pressionNon),2)),2);

    pressionGaz   = pressionIon+pressionElect; // pression du gaz
    pression      = pressionGaz+pressionRad;   // pression totale
    // pression doit tre de l'ordre de 10^17 dynes/cm2 au centre du soleil

    relativiste  = (5.0/3.0)*Math.pow((pressionDeg/pressionDegNR),2) + (4.0/3.0)*Math.pow((pressionDeg/pressionDegR),2);
    dLogPEdLogT   = Math.pow((pressionNon/pressionElect),2);
    dLogPEdLogD   = dLogPEdLogT+(1.0-dLogPEdLogT)*relativiste;

    dLogPdLogT    = (4.0*pressionRad+pressionIon + dLogPEdLogT*pressionElect)/pression;
    dLogPdLogD    = (pressionIon+dLogPEdLogD*pressionElect)/pression;
    qt            = (12.0*pressionRad+1.5*pressionIon+pressionElect*dLogPEdLogT/(relativiste-1.0))/d;
    qr            = pression*dLogPdLogT/d;
    dLnTdLnPad    = 1.0/(dLogPdLogD*qt/qr+dLogPdLogT); // DEL ADIABATIQUE
    dLnTdLnPrad   = constGrrad*kOpacite*pression/pressionRad; // DEL RAD*M/L

  }

/** Rsout les quations diffrentielles de la structure interne.<br>
    Entre : tableau de valeurs initiales<br>
    Sortie : tableau des drives, d Params(i)/d Params(1)<br>
    Les paramtres sont : temprature, densit, rayon, luminosit, masse
*/
  public Vecteur5 evalueEquations(Vecteur5 params){
  
    Vecteur5 deriv = new Vecteur5();

    // Variable(s) locale(s)
    // ---------------------
    double t;         // temprature
    double d;         // densit
    double r;         // rayon
    double l;         // Luminosit
    double m;         // masse
    double dLnTdLnP;  // gradient de temperature dans l'toile dLnt/dLnp
    double dLnDdLnP;  // gradient de densit dLnDensity/dLnp
    double dPdM;      // gradient de pression dLnp/d(M/Msun)

    t = params.p0; // temprature
    d = params.p1; // densit
    r = params.p2; // rayon
    l = params.p3; // Luminosit
    m = params.p4; // masse

    this.opacite(x,y,z,d,t);                  // on calcule d'abord l'opacit
    this.equationDEtat(x,y,z,d,t);            // calcul de l'quation d'tat
    this.energieNucleaire(x,y,z,zAzote,d,t);  // calcul de la production d'nergie
    dLnTdLnPrad = dLnTdLnPrad*l/m;     // DEL RAD
    
    // on prend le plus petit des gradients de T entre adiabatique et radiatif

    dLnTdLnP = Math.min(dLnTdLnPad,dLnTdLnPrad);

    dLnDdLnP = (1.-dLnTdLnP*dLogPdLogT)/dLogPdLogD;         // gradient de densit
    dPdM     = -constdPdM*m/Math.pow(r,4);                  // gradient de pression

    deriv.p0 = dLnTdLnP*dPdM*t/pression;              // dT/dM
    deriv.p1 = dLnDdLnP*dPdM*d/pression;              // dD/dM
    deriv.p2 = densiteSoleil/3.0/d/(r*r);             // dr/dM
    deriv.p3 = SOLEIL_MASSE/SOLEIL_LUM*puissance;     // dL/dM
    deriv.p4 = 1.;                                    // dM/dM
    
    return deriv;

  }

/** Intgration d'un point de grille au suivant par mthode de Runge-Kutta  l'ordre 4 des grandeurs
    passes en paramtre (de type Vecteur5).<br>
    Entre : valeurs initiales<br>
    Sortie : valeurs aprs intgration sur un pas de grille
*/
  public Vecteur5 integre1pas(Vecteur5 parameters) {
  
    // Variable(s) locale(s)
    // ---------------------
    Vecteur5 derivs = new Vecteur5();         // derives des paramtres
    Vecteur5 newParameters = new Vecteur5();  // valeurs aux points intermdiaires RK
    Vecteur5 k1 = new Vecteur5();
    Vecteur5 k2 = new Vecteur5();
    Vecteur5 k3 = new Vecteur5();
    Vecteur5 k4 = new Vecteur5();
    double pasInteg = 0;         // pas d'intgration

    derivs = evalueEquations(parameters);   // rsout les quations diffrentielles avec les paramtres
        
// Calcul du pas d'integration. Le pas d'intgration est le plus petit de ceux font varier les parametres de moins de maxparamstep en valeurs relatives

    double maxinter = 0;
    pasInteg = Math.abs(derivs.p0/maxParamStep[0]/parameters.p0);
    pasInteg = Math.max(pasInteg,maxinter);
    maxinter = pasInteg;
    pasInteg = Math.abs(derivs.p1/maxParamStep[1]/parameters.p1);
    pasInteg = Math.max(pasInteg,maxinter);
    maxinter = pasInteg;
    pasInteg = Math.abs(derivs.p2/maxParamStep[2]/parameters.p2);
    pasInteg = Math.max(pasInteg,maxinter);
    maxinter = pasInteg;
    pasInteg = Math.abs(derivs.p3/maxParamStep[3]/parameters.p3);
    pasInteg = Math.max(pasInteg,maxinter);
    maxinter = pasInteg;
    pasInteg = Math.abs(derivs.p4/maxParamStep[4]/parameters.p4);
    pasInteg = Math.max(pasInteg,maxinter);
    
    pasInteg = 1./pasInteg;
    
    if (parameters.p4 > fitMass) {
       pasInteg = -pasInteg;  // pas ngatif si masse > masse au point de jonction
    }

    if ((parameters.p4-fitMass)*(parameters.p4+pasInteg-fitMass) < 0.0) {
       pasInteg = fitMass-parameters.p4;  // pour le point le plus proche du point de jonction
    }

// Calcul de k1

    k1.p0 = pasInteg*derivs.p0;
    k1.p1 = pasInteg*derivs.p1;
    k1.p2 = pasInteg*derivs.p2;
    k1.p3 = pasInteg*derivs.p3;
    k1.p4 = pasInteg*derivs.p4;

// Calcul de k2

      newParameters.p0 = parameters.p0 + k1.p0/2.;
      newParameters.p1 = parameters.p1 + k1.p1/2.;
      newParameters.p2 = parameters.p2 + k1.p2/2.;
      newParameters.p3 = parameters.p3 + k1.p3/2.;
      newParameters.p4 = parameters.p4 + k1.p4/2.;
      
      derivs = evalueEquations(newParameters);
      
      k2.p0 = pasInteg*derivs.p0;
      k2.p1 = pasInteg*derivs.p1;
      k2.p2 = pasInteg*derivs.p2;
      k2.p3 = pasInteg*derivs.p3;
      k2.p4 = pasInteg*derivs.p4;
      
// Calcul de k3

      newParameters.p0 = parameters.p0 + k2.p0/2.;
      newParameters.p1 = parameters.p1 + k2.p1/2.;
      newParameters.p2 = parameters.p2 + k2.p2/2.;
      newParameters.p3 = parameters.p3 + k2.p3/2.;
      newParameters.p4 = parameters.p4 + k2.p4/2.;
      
      derivs = evalueEquations(newParameters);
      
      k3.p0 = pasInteg*derivs.p0;
      k3.p1 = pasInteg*derivs.p1;
      k3.p2 = pasInteg*derivs.p2;
      k3.p3 = pasInteg*derivs.p3;
      k3.p4 = pasInteg*derivs.p4;
      
// Calcul de k4

      newParameters.p0 = parameters.p0 + k3.p0;
      newParameters.p1 = parameters.p1 + k3.p1;
      newParameters.p2 = parameters.p2 + k3.p2;
      newParameters.p3 = parameters.p3 + k3.p3;
      newParameters.p4 = parameters.p4 + k3.p4;
      
      derivs = evalueEquations(newParameters);
      
      k4.p0 = pasInteg*derivs.p0;
      k4.p1 = pasInteg*derivs.p1;
      k4.p2 = pasInteg*derivs.p2;
      k4.p3 = pasInteg*derivs.p3;
      k4.p4 = pasInteg*derivs.p4;
      
    // calcul final
    
    parameters.p0 = parameters.p0 + (k2.p0+k3.p0)/3.0 + (k1.p0+k4.p0)/6.0;
    parameters.p1 = parameters.p1 + (k2.p1+k3.p1)/3.0 + (k1.p1+k4.p1)/6.0;
    parameters.p2 = parameters.p2 + (k2.p2+k3.p2)/3.0 + (k1.p2+k4.p2)/6.0;
    parameters.p3 = parameters.p3 + (k2.p3+k3.p3)/3.0 + (k1.p3+k4.p3)/6.0;
    parameters.p4 = parameters.p4 + (k2.p4+k3.p4)/3.0 + (k1.p4+k4.p4)/6.0;
    
    return parameters;

  }

/** Intgration de la structure stellaire depuis le centre de l'toile.<br>
    Entre : temprature et densit au centre<br>
    Sortie : les 5 valeurs au point de jonction (T, densit, rayon, luminosit, masse)
*/
  public Vecteur5 dehors(double tCent,double dCent,int error){

    Vecteur5 core = new Vecteur5();

    // Constantes(s) locale(s)
    // -----------------------
    final int maxiteration = 1000;    // nombre maximum d'itrations

    // Variable(s) locale(s)
    // ---------------------
    int iteration;           // compteur d'itrations
    double deviation;        // cart de la masse au dernier point  la masse au point de jonction
    double masseCoeur;       // masse du coeur (en masse solaire)
    double luminositeCoeur;  // luminosit du coeur (en luminosit solaire)
    double xx, yy;
    int index; 

    this.energieNucleaire(x,y,z,zAzote,dCent,tCent); // calcul de la production d'nergie
    masseCoeur        = fitMass/10000.0;              // masse du coeur  partir de laquelle on intgre
    luminositeCoeur  = SOLEIL_MASSE/SOLEIL_LUM*masseCoeur*puissance;
    rayonCoeur      = Math.pow((masseCoeur/dCent*densiteSoleil),(1.0/3.0));

    // initialisation des 5 paramtres au coeur
    core.p0 = tCent;             // temprature du coeur
    core.p1 = dCent;             // densit
    core.p2 = rayonCoeur;        // rayon (en rayon solaire)
    core.p3 = luminositeCoeur;   // Luminosit (en luminosit solaire)
    core.p4 = masseCoeur;        // masse du coeur (en masse solaire)

    iteration = 0;
   
    do {
       iteration = iteration +1;                    // compteur d'itrations
       core = integre1pas(core);                    // on intgre
       deviation = Math.abs((core.p4/fitMass)-1.0); // cart  la masse au point de jonction
       core.p0 = Math.max(core.p0,tEffective);      // pour toujours avoir T>0
       core.p1 = Math.max(core.p1,1.0e-12);         // pour toujours avoir  rho>0

       if(plotControl == 1) {

          if(plotMasseOuRayon) {
             xx = core.p2/radius;  // pour des tracs en fonction du rayon
          }
          else {
             xx = core.p4/mass;    // pour des tracs en fonction de la masse
          }
          if((xx >0.001) && (xx < 0.999)) {
             vecteurXout.addElement(new Double(xx));
             yy = Math.log(core.p0)/Math.log(10);
             vecteurTout.addElement(new Double(yy));
             yy = Math.log(core.p1)/Math.log(10);
             vecteurDout.addElement(new Double(yy));
             if(plotMasseOuRayon) {
               yy = Math.log(core.p4)/Math.log(10);
             }
             else {
               yy = Math.log(core.p2)/Math.log(10);
             }
             vecteurRout.addElement(new Double(yy));
             yy = Math.log(core.p3)/Math.log(10);
             vecteurLout.addElement(new Double(yy));

             index = (int) Math.floor(xx*keepSize); // indice de tableau
             T_array[index] = core.p0;
             D_array[index] = core.p1;
             R_array[index] = core.p2;
             L_array[index] = core.p3;
             M_array[index] = core.p4;
             K_array[index] = kOpacite;
             E_array[index] = puissance;
             P_array[index] = pression;
          }
       }
    }
    while((deviation > fitPrecision) && (iteration < maxiteration));  // fin de la boucle do...while
    
    // le point de jonction n'est pas atteint aprs maxIteration itrations
    if( deviation > fitPrecision) {
       System.out.println("dehors:trop loin de la jonction. Dviation=" + deviation);
    }
    
    return core;
  }

/** Intgration de la structure stellaire depuis la surface de l'toile.<br>
    Entre : temprature effective et luminosit<br>
    Sortie : les 5 valeurs au point de jonction (T, densit, rayon, luminosit, masse)
*/
  public Vecteur5 dedans(double tEff, double lum, int error){

    Vecteur5 surf = new Vecteur5();

    // Constantes(s) locale(s)
    // -----------------------
    final int maxiteration = 1000;    // nombre maximum d'itrations

    // Variable(s) locale(s)
    // ---------------------
    int iteration;           // compteur d'itrations
    double deviation;        // cart de la masse au dernier point  la masse au point de jonction
    double xx, yy;
    int index;
    
    radius = Math.sqrt(lum)/Math.pow((tEff/SOLEIL_T_SURFACE),2); // rayon de la photosphre (en rayon solaire)

    // initialisation des 5 paramtres  la surface
    surf.p0 = tEff/twoTo14; // temp de surface plus petite que Teffect
    surf.p1 = 1.0e-12;      // densit trs faible
    surf.p2 = radius;       // rayon (en rayon solaire)
    surf.p3 = lum;          // Luminosit (en luminosit solaire)
    surf.p4 = mass;         // masse du coeur (en masse solaire)

    iteration = 0;

    do { 
       iteration = iteration +1;                    // compteur d'itrations
       surf = integre1pas(surf);                    // on intgre
       deviation = Math.abs((surf.p4/fitMass)-1.0); // cart  la masse au point de jonction
       surf.p3 = Math.max(surf.p3,1.0e-10);         // pour toujours avoir L>0

       if(plotControl == 1) {

          if(plotMasseOuRayon) { 
             xx = surf.p2/radius;  // pour des tracs en fonction du rayon
          }
          else {
             xx = surf.p4/mass;    // pour des tracs en fonction de la masse
          }
          if((xx > 0.001) && (xx < 0.999)) {
             vecteurXin.addElement(new Double(xx));
             yy = Math.log(surf.p0)/Math.log(10);
             vecteurTin.addElement(new Double(yy));
             yy = Math.log(surf.p1)/Math.log(10);
             vecteurDin.addElement(new Double(yy));
             if(plotMasseOuRayon) {
               yy = Math.log(surf.p4)/Math.log(10);
             }
             else {
               yy = Math.log(surf.p2)/Math.log(10);
             }
             vecteurRin.addElement(new Double(yy));
             yy = Math.log(surf.p3)/Math.log(10);
             vecteurLin.addElement(new Double(yy));

             index = (int) Math.floor(xx*keepSize); // indice de tableau
             T_array[index] = surf.p0;
             D_array[index] = surf.p1;
             R_array[index] = surf.p2;
             L_array[index] = surf.p3;
             M_array[index] = surf.p4;
             K_array[index] = kOpacite;
             E_array[index] = puissance;
             P_array[index] = pression;
          }
       }
    }
    while((deviation > fitPrecision) && (iteration < maxiteration));  // fin de la boucle do...while

    // le point de jonction n'est pas atteint aprs maxIteration itrations
    if( deviation > fitPrecision) {
       System.out.println("dedans:trop loin de la jonction. Dviation=" + deviation);
    }
    
    return surf;    
  }

/** Correction des conditions aux limites pour l'itration suivante.<br>
    Entre : anciennes conditions aux limites et valeurs des drives<br>
    Sortie : valeur corrige pour Teff, densit" et temprature au coeur, luminosit<br>
    Cette mthode utilise la technique de BP pour trouver les corrections.
*/
  public void correctionValeur(double deriv[][]){

    // Constantes(s) locale(s)
    // -----------------------
    final int numEqs=4; // nombre d'quations

    // Variable(s) locale(s)
    // ---------------------
    int n=numEqs;     //  des fins de comparaison vs. Golub/Van Loan
    int mu;           // position du pivot d'ordre k
    int pivk;         // position du pivot au pas k
    double buffer;
    double factor;    // facteur de rduction pour les corrections, ou bien correction max
    double maxmat;    // valeur max. de la sous-matrice pour determiner mu
 
    // rsolution d'un systme de 4 quations pour T,rho,r,L: sommation sur j:
    //        deriv(i,j)*corrections(j)+deriv(i,5)=0

    // LU decomposition (Golub/Van Loan p.112) n=NumEqs
    mu=0;
    for( int k=0 ; k <= n-2 ; k++ ) {
      maxmat=0;
      for( int i=k ; i <= n-1 ; i++ ) {
        for ( int j=0 ; j <= k ;  j++ ) {
          if (Math.abs(deriv[i][j]) >= maxmat) {
            maxmat = Math.abs(deriv[i][j]);
            mu=i-k;
          }
        }
      }
      pivk = mu + k;
       // permutation des Lignes k/pivk
      if (pivk != k) {
        for ( int i=k ; i <= n ; i++ ) {
          buffer = deriv[k][i];
          deriv[k][i] = deriv[pivk][i];
          deriv[pivk][i] = buffer;
        }
      }
      if (deriv[k][k] != 0.) {
        for ( int i=k+1 ; i <= n-1 ; i++ ) {
          deriv[i][k] = deriv[i][k]/deriv[k][k];
        }
        for ( int i=k+1 ; i <= n-1 ; i++ ) {
          for ( int j=k+1 ; j <= n ; j++ ) {
            deriv[i][j] = deriv[i][j]-deriv[i][k]*deriv[k][j];
          }
        }
      }
      else {
        System.out.println("correctionValeur: division par 0 dans calcul de deriv[i][j]");
      }
    }

    // on introduit le second membre, qui est -delta(:)
    for ( int i=0 ; i <= n-1 ; i++ ) {
      corrections[i] = -deriv[i][n];
    }
    // Back Substitution (Golub/Van Loan p.89)
    if ( deriv[n-1][n-1] != 0. ) {
      corrections[n-1] = corrections[n-1]/deriv[n-1][n-1];
    }
    else {
      System.out.println("correctionValeur: division par 0 dans calcul de corrections[n-1]");
    }
    for (int i=n-2 ; i >= 0 ; i-- ) {
      double dot_product = 0;
      for (int j=i+1 ; j<= n-1 ; j++) {
        dot_product=dot_product + deriv[i][j]*corrections[j];
      }
      if ( deriv[i][i] != 0. ) {
        corrections[i] = (corrections[i]-dot_product)/deriv[i][i];
      }
      else {
        System.out.println("correctionValeur: division par 0 dans calcul de corrections[i]");
      }
    }
    // les corrections inconnues del(z(i)) ont t trouves

    // vrification de la taille des corrections
    double maxinter = 0;
    factor=1;
    for (int i=0 ; i <= numEqs-1 ; i++) {
      factor = Math.abs(corrections[i]/maxCorrect[i]);
      factor = Math.max(factor,maxinter);
      maxinter = factor;
      factor = Math.max(1.,factor);
    }

    for (int i=0 ; i <= numEqs-1 ; i++) {
      corrections[i] = corrections[i]/factor;
    }

    // on utilise les del(z(i)) pour corriger les conditions aux limites
    tEffective  = tEffective*(1+corrections[0]);
    luminosite  = luminosite*(1+corrections[1]);
    tCentral    = tCentral*(1+corrections[2]);
    dCentral    = dCentral*(1+corrections[3]);

  }

}

class Vecteur5 {
  double p0;
  double p1;
  double p2;
  double p3;
  double p4;
}
