/*
Etoile - algorithme de simulation pour SimuApplet

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;
import java.util.*;

/** Algorithme de simulation pour l'environnement SimuApplet.
    Calcule une structure d'toile sur la Squence Principale d'ge Zro (toile chimiquement homogne).
*/
public class Etoile extends SimuApplet {

/*!  +-----------------------------------+
  !  | Dclaration des variables globales |
  !  +-----------------------------------+*/

  boolean apprendre;
  boolean nouveauCalcul;
  boolean ptJctAuto;

  double mass;         // Masse de l'toile
  double luminosite;   // Luminosit de surface
  double tEffective;   // Temprature effective
  double tCentral;     // Temprature centrale
  double dCentral;     // Densit centrale
  double tZero;        // "vraie" temprature au centre (avec extrapolation AR)
  double dZero;        // "vraie" densit au centre (d)
  double fitMassAuto;  // Masse au point de jonction calcul automatiquement
  double minDeltaGlob; // Le minimum maximorum pour recherche auto. du point de jonction

  double fitMassFraction; // Fraction de masse au point de jonction

  double coreVals[] = new double[5];      // Valeurs des paramtres au point d'ajustement calcules  partir du centre
  double surfaceVals[] = new double[5];   // Valeurs des paramtres au point d'ajustement calcules  partir de la surface
  double coreValsT[] = new double[5];     // Valeurs  partir du centre avec variation de T
  double surfaceValsT[] = new double[5];  // Valeurs  partir de la surface avec variation de T
  double coreValsD[] = new double[5];     // Valeurs  partir du centre avec variation de densit
  double surfaceValsL[] = new double[5];  // Valeurs  partir de la surface avec variation de luminosit

  double derivitive[][] = new double[5][5];  // Drives des valeurs au point de jonction par rapport  une variation de T, D...
  
  int nIter;
  boolean finIter;     // Pour indiquer qu'une itration est acheve
  boolean finCalcul;   // Pour indiquer que le calcul est termin
  Biblietoile bibli;   // C'est avec un objet de la classe Biblietoile que le calcul est effectu
    
/** Constructeur de la classe, avec la dfinition des paramtres de simulation
    et des options d'affichage.
*/
    public Etoile() {
        nIter = 0;
        // paramtres d'entre
        ParamIn masse = new ParamIn();
        masse.type = "nombre";
        masse.label = "masse";
        masse.titre = "Masse de l'toile";
        masse.unit = "masse solaire";
        masse.acquisition = "champ";
        masse.defaut = "1";
        addParamIn(masse);
        
        ParamIn tEff = new ParamIn();
        tEff.type = "nombre";
        tEff.label = "tEff";
        tEff.titre = "Temprature effective";
        tEff.unit = "K";
        tEff.acquisition = "champ";
        tEff.defaut = "5777";
        addParamIn(tEff);
        
        ParamIn luminosite = new ParamIn();
        luminosite.type = "nombre";
        luminosite.label = "luminosite";
        luminosite.titre = "Luminosit";
        luminosite.unit = "luminosit solaire";
        luminosite.acquisition = "champ";
        luminosite.defaut = "0.5";
        addParamIn(luminosite);
        
        ParamIn tCentre = new ParamIn();
        tCentre.type = "nombre";
        tCentre.label = "tCentre";
        tCentre.titre = "Temprature centrale";
        tCentre.unit = "K";
        tCentre.acquisition = "champ";
        tCentre.defaut = "1.3E7";
        addParamIn(tCentre);
        
        ParamIn dCentre = new ParamIn();
        dCentre.type = "nombre";
        dCentre.label = "dCentre";
        dCentre.titre = "Densit centrale";
        dCentre.unit = "cgs";
        dCentre.acquisition = "champ";
        dCentre.defaut = "65.53";
        addParamIn(dCentre);
        
        ParamIn masseJ = new ParamIn();
        masseJ.type = "nombre";
        masseJ.label = "masseJ";
        masseJ.titre = "Fract. de masse au pt de jonction";
        masseJ.acquisition = "champ";
        masseJ.defaut = "0.5";
        addParamIn(masseJ);
        
        ParamIn learn = new ParamIn();
        learn.type = "string";
        learn.label = "apprendre";
        learn.titre = "Mode apprentissage";
        learn.acquisition = "case";
        learn.defaut = "off";
        addParamIn(learn);
        
        ParamIn ptJct = new ParamIn();
        ptJct.type = "string";
        ptJct.label = "jonction";
        ptJct.titre = "Point de jonction auto.";
        ptJct.acquisition = "case";
        ptJct.defaut = "off";
        addParamIn(ptJct);
        
        // dfinition de deux ensembles de valeurs des paramtres
        setTitreEnsembles("Modle de dpart");
        ListeValeurs l1 = new ListeValeurs();
        l1.ajouterString("masse", "1");
        l1.ajouterString("tEff", "5940");
        l1.ajouterString("luminosite", "0.5");
        l1.ajouterString("tCentre", "1.3e7");
        l1.ajouterString("dCentre", "65.5");
        addEnsemble("1", l1);

        ListeValeurs l2 = new ListeValeurs();
        l2.ajouterString("masse", "2");
        l2.ajouterString("tEff", "11460");
        l2.ajouterString("luminosite", "15.3");
        l2.ajouterString("tCentre", "2.1e7");
        l2.ajouterString("dCentre", "61.9");
        addEnsemble("2", l2);

        ListeValeurs l5 = new ListeValeurs();
        l5.ajouterString("masse", "5");
        l5.ajouterString("tEff", "21200");
        l5.ajouterString("luminosite", "688");
        l5.ajouterString("tCentre", "2.7E7");
        l5.ajouterString("dCentre", "20.7");
        addEnsemble("5", l5);
      
        ListeValeurs l10 = new ListeValeurs();
        l10.ajouterString("masse", "10");
        l10.ajouterString("tEff", "30070");
        l10.ajouterString("luminosite", "6990");
        l10.ajouterString("tCentre", "3.2E7");
        l10.ajouterString("dCentre", "9.1");
        addEnsemble("10", l10);
           
        ListeValeurs l20 = new ListeValeurs();
        l20.ajouterString("masse", "20");
        l20.ajouterString("tEff", "40000");
        l20.ajouterString("luminosite", "49080");
        l20.ajouterString("tCentre", "3.5E7");
        l20.ajouterString("dCentre", "4.7");
        addEnsemble("20", l20);
           
        ListeValeurs l50 = new ListeValeurs();
        l50.ajouterString("masse", "50");
        l50.ajouterString("tEff", "53340");
        l50.ajouterString("luminosite", "392500");
        l50.ajouterString("tCentre", "4.0E7");
        l50.ajouterString("dCentre", "2.4");
        addEnsemble("50", l50);
           
        // paramtres de sortie
        ParamOut m_out = new ParamOut();
        m_out.type = "nombre";
        m_out.label = "m_out";
        m_out.titre = "masse";
        addParamOut(m_out);
        
        ParamOut t_out = new ParamOut();
        t_out.type = "nombre";
        t_out.label = "t_out";
        t_out.titre = "log T";
        addParamOut(t_out);
        
        ParamOut d_out = new ParamOut();
        d_out.type = "nombre";
        d_out.label = "d_out";
        d_out.titre = "log D";
        addParamOut(d_out);
        
        ParamOut r_out = new ParamOut();
        r_out.type = "nombre";
        r_out.label = "r_out";
        r_out.titre = "log R";
        addParamOut(r_out);
        
        ParamOut l_out = new ParamOut();
        l_out.type = "nombre";
        l_out.label = "l_out";
        l_out.titre = "log L";
        addParamOut(l_out);
        
        ParamOut m_in = new ParamOut();
        m_in.type = "nombre";
        m_in.label = "m_in";
        m_in.titre = "masse";
        addParamOut(m_in);
        
        ParamOut t_in = new ParamOut();
        t_in.type = "nombre";
        t_in.label = "t_in";
        t_in.titre = "log T";
        addParamOut(t_in);
        
        ParamOut d_in = new ParamOut();
        d_in.type = "nombre";
        d_in.label = "d_in";
        d_in.titre = "log D";
        addParamOut(d_in);
        
        ParamOut r_in = new ParamOut();
        r_in.type = "nombre";
        r_in.label = "r_in";
        r_in.titre = "log R";
        addParamOut(r_in);
        
        ParamOut l_in = new ParamOut();
        l_in.type = "nombre";
        l_in.label = "l_in";
        l_in.titre = "log L";
        addParamOut(l_in);
        
        ParamOut tEff2 = new ParamOut();
        tEff2.type = "nombre";
        tEff2.label = "tEff";
        tEff2.titre = "Temprature effective";
        addParamOut(tEff2);
        
        ParamOut luminosite2 = new ParamOut();
        luminosite2.type = "nombre";
        luminosite2.label = "luminosite";
        luminosite2.titre = "Luminosit";
        addParamOut(luminosite2);
        
        ParamOut tCentre2 = new ParamOut();
        tCentre2.type = "nombre";
        tCentre2.label = "tCentre";
        tCentre2.titre = "Temprature centrale";
        addParamOut(tCentre2);
        
        ParamOut dCentre2 = new ParamOut();
        dCentre2.type = "nombre";
        dCentre2.label = "dCentre";
        dCentre2.titre = "Densit centrale";
        addParamOut(dCentre2);
        
        ParamOut masseJ2 = new ParamOut();
        masseJ2.type = "nombre";
        masseJ2.label = "masseJ";
        masseJ2.titre = "Fract. de masse au pt de jonction";
        addParamOut(masseJ2);
        
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
        
        // options d'affichage
        
        // affichage "multiplot"
        Affichage multiM = new Affichage();
        multiM.type = "multiplot";
        multiM.boucle = true;  // pour mettre les parametres de sortie  en entree
        multiM.titre = "Calcul d'une structure stellaire";
        multiM.sousAff = new Affichage[4];
        
        Affichage m1M = new Affichage();
        m1M.type = "plotlive";
        m1M.titre = "Temprature";
        m1M.points = "none";
        m1M.xdebut = "0";
        m1M.xfin = "1";
        String[][] pparamsm1M = {{"m_out", "t_out"},{"m_in", "t_in"}};
        m1M.params = pparamsm1M;
        m1M.connecter = true;
        multiM.sousAff[0] = m1M;
        
        Affichage m2M = new Affichage();
        m2M.type = "plotlive";
        m2M.titre = "Densit";
        m2M.points = "none";
        m2M.xdebut = "0";
        m2M.xfin = "1";
        String[][] pparamsm2M = {{"m_out", "d_out"},{"m_in", "d_in"}};
        m2M.params = pparamsm2M;
        m2M.connecter = true;
        multiM.sousAff[1] = m2M;
        
        Affichage m3M = new Affichage();
        m3M.type = "plotlive";
        m3M.titre = "Rayon";
        m3M.points = "none";
        m3M.xdebut = "0";
        m3M.xfin = "1";
        String[][] pparamsm3M = {{"m_out", "r_out"},{"m_in", "r_in"}};
        m3M.params = pparamsm3M;
        m3M.connecter = true;
        multiM.sousAff[2] = m3M;
        
        Affichage m4M = new Affichage();
        m4M.type = "plotlive";
        m4M.titre = "Luminosit";
        m4M.points = "none";
        m4M.xdebut = "0";
        m4M.xfin = "1";
        String[][] pparamsm4M = {{"m_out", "l_out"},{"m_in", "l_in"}};
        m4M.params = pparamsm4M;
        m4M.connecter = true;
        multiM.sousAff[3] = m4M;
        
        addAffichage(multiM);

/*        // affichage "multiplot" avec tracs en fonction du rayon.
        Affichage multiR = new Affichage();
        multiR.type = "multiplot";
        multiR.boucle = true;  // pour mettre les parametres de sortie  en entree
        multiR.titre = "Calcul d'une structure stellaire";
        multiR.sousAff = new Affichage[4];
        
        Affichage m1R = new Affichage();
        m1R.type = "plotlive";
        m1R.titre = "Temprature";
        m1R.points = "none";
        m1R.xdebut = "0";
        m1R.xfin = "1";
        String[][] pparamsm1R = {{"r_out", "t_out"},{"r_in", "t_in"}};
        m1R.params = pparamsm1R;
        m1R.connecter = true;
        multiR.sousAff[0] = m1R;
        
        Affichage m2R = new Affichage();
        m2R.type = "plotlive";
        m2R.titre = "Densit";
        m2R.points = "none";
        m2R.xdebut = "0";
        m2R.xfin = "1";
        String[][] pparamsm2R = {{"r_out", "d_out"},{"r_in", "d_in"}};
        m2R.params = pparamsm2R;
        m2R.connecter = true;
        multiR.sousAff[1] = m2R;
        
        Affichage m3R = new Affichage();
        m3R.type = "plotlive";
        m3R.titre = "Masse";
        m3R.points = "none";
        m3R.xdebut = "0";
        m3R.xfin = "1";
        String[][] pparamsm3R = {{"r_out", "m_out"},{"r_in", "m_in"}};
        m3R.params = pparamsm3R;
        m3R.connecter = true;
        multiR.sousAff[2] = m3R;
        
        Affichage m4R = new Affichage();
        m4R.type = "plotlive";
        m4R.titre = "Luminosit";
        m4R.points = "none";
        m4R.xdebut = "0";
        m4R.xfin = "1";
        String[][] pparamsm4R = {{"r_out", "l_out"},{"r_in", "l_in"}};
        m4R.params = pparamsm4R;
        m4R.connecter = true;
        multiR.sousAff[3] = m4R;
        
        addAffichage(multiR);*/

    appletDoc("stellardoc.xml");
    bibli = new Biblietoile();
  }
    
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
  public void initCalculLive(ListeValeurs in) throws SimuException {
  
    mass = in.lireDouble("masse");
    tEffective = in.lireDouble("tEff");
    luminosite = in.lireDouble("luminosite");
    tCentral = in.lireDouble("tCentre");
    dCentral = in.lireDouble("dCentre");
    fitMassFraction = in.lireDouble("masseJ");
    apprendre = (in.lireString("apprendre") == "on");
    ptJctAuto = (in.lireString("jonction") == "on");
    nouveauCalcul = true;

// on transmet les valeurs a bibli

    bibli.mass = in.lireDouble("masse");
    bibli.tEffective = in.lireDouble("tEff");
    bibli.luminosite = in.lireDouble("luminosite");
    bibli.tCentral = in.lireDouble("tCentre");
    bibli.dCentral = in.lireDouble("dCentre");
    bibli.fitMass = fitMassFraction * mass;
    bibli.initialiseModele();
    
// pour ajuster l'chelles des ordonnes en fonction des valeurs du modle de dpart
    Vector affichages = getAffichage();
    Affichage sous_aff[] = new Affichage[4];
    for ( int k=0 ; k < affichages.size() ; k++ ) {
      for ( int l=0 ; l < ((Affichage)affichages.elementAt(k)).sousAff.length ; l++ ) {
        sous_aff[l] = ((Affichage)affichages.elementAt(k)).sousAff[l];
        if ( sous_aff[l].titre == "Temprature" ) {
          sous_aff[l].ydebut = Double.toString(Math.log(tEffective)/Math.log(10)+2.);
          sous_aff[l].yfin = Double.toString(Math.log(tCentral)/Math.log(10)+0.5);
        }
        if ( sous_aff[l].titre == "Densit" ) {
          sous_aff[l].ydebut = Double.toString(Math.log(dCentral)/Math.log(10)-2.);
          sous_aff[l].yfin = Double.toString(Math.log(dCentral)/Math.log(10)+0.5);
        }
        if ( sous_aff[l].titre == "Rayon" ) {
          sous_aff[l].ydebut = Double.toString(Math.log(bibli.radius)/Math.log(10)-1.);
          sous_aff[l].yfin = Double.toString(Math.log(bibli.radius)/Math.log(10)+0.3);
        }
        if ( sous_aff[l].titre == "Masse" ) {
          sous_aff[l].ydebut = Double.toString(Math.log(bibli.radius)/Math.log(10)-1.);
          sous_aff[l].yfin = Double.toString(Math.log(bibli.radius)/Math.log(10)+0.3);
        }
        if ( sous_aff[l].titre == "Luminosit" ) {
          sous_aff[l].ydebut = Double.toString(Math.log(luminosite)/Math.log(10)-1.);
          sous_aff[l].yfin = Double.toString(Math.log(luminosite)/Math.log(10)+0.5);
        }
      }
    }
  }
    
    // Calcul du temps t au temps t+1.
  public ListeValeurs calculLive() throws SimuException {
  
// Constantes locales
    final int maxiteration = 20;  //1000;  // nombre maximal d'itrations
    final double tCdel = 0.0011;    // incrment de temprature centrale pour calcul de la drive au point de jonction
    final double tSdel = 0.0011;    // incrment de temprature de surface pour calcul de la drive au point de jonction
    final double dCdel = 0.0011;    // incrment de densit centrale pour calcul de la drive au point de jonction
    final double lSdel = 0.0011;    // incrment de luminosit de surface pour calcul de la drive au point de jonction

// Variables locales
    double maxDelta;              // la plus grande diffrence au point de jonction pour l'itration en cours
    double delta[]={0,0,0,0,0};   // diffrences entre valeurs  partir du centre et  partir de la surface
    Vecteur5 tempo;
    
    int error = 0;
    String message;
    
    ListeValeurs out = new ListeValeurs();

// les contrles: on passe ici un appel  calculLive sur 2. Les autres fois, on fait le calcul proprement dit
    if ( apprendre && nIter >= 1 ) {
      out.ajouterString("fin", "simulation");
      nIter = 0;   // On initialise nIter  0 pour l'itration suivante
      return(out);
    }
// le calcul est termin, on sort les valeurs du modle pour le calcul suivant
    if ( finCalcul ) {
      out.ajouterString("fin", "simulation");
      nIter = 0;          // On initialise nIter  0 pour le calcul suivant
      finCalcul = false;  // De mme, initialisation
      out.ajouterDouble("tEff", tEffective);  // On sort les valeurs du modle pour le calcul suivant
      out.ajouterDouble("dCentre", dCentral);
      out.ajouterDouble("tCentre", tCentral);
      out.ajouterDouble("luminosite", luminosite);
      out.ajouterDouble("masseJ", fitMassFraction);
      return(out);
    }
// l'itration est termine, on sort les valeurs du modle pour l'itration suivante
    if ( finIter ) {
      out.ajouterString("fin", "trajectoire");
      finIter = false;
      out.ajouterDouble("tEff", tEffective);
      out.ajouterDouble("dCentre", dCentral);
      out.ajouterDouble("tCentre", tCentral);
      out.ajouterDouble("luminosite", luminosite);
      out.ajouterDouble("masseJ", fitMassFraction);
      return(out);
    }

    nIter++;
    System.out.println("itration:"+nIter);
    
// pour calcul automatique du point de jonction
    if ( ptJctAuto ) {
  
      System.out.println("On dtermine le point de jonction");
      minDeltaGlob = 1e10;  // On met une grande valeur en esprant qu'elle ne sera jamais atteinte
      for ( int i=0 ; i < 9 ; i++ ) {
    
        fitMassFraction = (0.1+i/10.);  // On fait un calcul tous les diximes de masse totale
        bibli.fitMass = mass * fitMassFraction;
    
        tempo = bibli.dehors(tCentral,dCentral,error);  // On intgre du centre au point de jonction

        coreVals[0] = tempo.p0;
        coreVals[1] = tempo.p1;
        coreVals[2] = tempo.p2;
        coreVals[3] = tempo.p3;
        coreVals[4] = tempo.p4;
      
        tempo = bibli.dedans(tEffective,luminosite,error);  // On intgre de la surface au point de jonction

        surfaceVals[0] = tempo.p0;
        surfaceVals[1] = tempo.p1;
        surfaceVals[2] = tempo.p2;
        surfaceVals[3] = tempo.p3;
        surfaceVals[4] = tempo.p4;
    
        maxDelta = 0;
        for ( int j=0 ; j <= 4 ; j++ ) {
          delta[j] = (surfaceVals[j]/coreVals[j])-1.0;
          if ( maxDelta < Math.abs(delta[j]) ) {
            maxDelta = Math.abs(delta[j]);  // Le maximum des diffrences relatives entre valeurs obtenues depuis la surface et depuis le centre
          }
        }
        if ( maxDelta < minDeltaGlob ) {
          minDeltaGlob = maxDelta;  // On dtermine le point o la diffrence maximale est la plus faible
          fitMassAuto = bibli.fitMass;
        }
      }      
      bibli.fitMass = fitMassAuto;  // On range le point dtermin ci-dessus dans fitMass
      fitMassFraction = bibli.fitMass/mass;
    }

// on calcule les profils sortant et entrant
    bibli.plotControl = 1;  // Pour stocker les valeurs du profil
    tempo = bibli.dehors(tCentral,dCentral,error);  // On intgre du centre au point de jonction

    coreVals[0] = tempo.p0;
    coreVals[1] = tempo.p1;
    coreVals[2] = tempo.p2;
    coreVals[3] = tempo.p3;
    coreVals[4] = tempo.p4;

    tempo = bibli.dedans(tEffective,luminosite,error);  // On intgre de la surface au point de jonction
    bibli.plotControl = 0;  // On ne  veut plus stocker les valeurs du profil
    
    surfaceVals[0] = tempo.p0;
    surfaceVals[1] = tempo.p1;
    surfaceVals[2] = tempo.p2;
    surfaceVals[3] = tempo.p3;
    surfaceVals[4] = tempo.p4;
    
// on calcule l'cart maximal
    maxDelta = 0;
    for ( int i=0 ; i <= 4 ; i++ ) {
      delta[i] = (surfaceVals[i]/coreVals[i])-1.0;
      if ( maxDelta < Math.abs(delta[i]) ) {
        maxDelta = Math.abs(delta[i]);  // Le maximum des diffrences relatives entre valeurs obtenues depuis la surface et depuis le centre
      }
    }
    
// quand on est en mode automatique, on calcule les drives des carts au point de jonction par rapport  un variation des conditions aux limites
    if ( !apprendre ) {

      tempo = bibli.dehors(tCentral*(1.+tCdel),dCentral,error);  // Variation de la temprature centrale
      
      coreValsT[0] = tempo.p0;
      coreValsT[1] = tempo.p1;
      coreValsT[2] = tempo.p2;
      coreValsT[3] = tempo.p3;
      coreValsT[4] = tempo.p4;

      tempo = bibli.dedans(tEffective*(1.+tSdel),luminosite,error);  // Variation de la temprature effective
      
      surfaceValsT[0] = tempo.p0;
      surfaceValsT[1] = tempo.p1;
      surfaceValsT[2] = tempo.p2;
      surfaceValsT[3] = tempo.p3;
      surfaceValsT[4] = tempo.p4;

      tempo = bibli.dehors(tCentral,dCentral*(1.+dCdel),error);  // Variation de la densit centrale

      coreValsD[0] = tempo.p0;
      coreValsD[1] = tempo.p1;
      coreValsD[2] = tempo.p2;
      coreValsD[3] = tempo.p3;
      coreValsD[4] = tempo.p4;

      tempo = bibli.dedans(tEffective,luminosite*(1.+lSdel),error);  // Variation de la luminosit

      surfaceValsL[0] = tempo.p0;
      surfaceValsL[1] = tempo.p1;
      surfaceValsL[2] = tempo.p2;
      surfaceValsL[3] = tempo.p3;
      surfaceValsL[4] = tempo.p4;

// Calcul des drives
      for ( int i=0 ; i <= 4 ; i++) {
        if ( coreVals[i] != 0.) {
          derivitive[i][0] = ((surfaceValsT[i]/coreVals[i]-1.0)-delta[i])/(tSdel);
          derivitive[i][1] = ((surfaceValsL[i]/coreVals[i]-1.0)-delta[i])/(lSdel);
        }
        else {
          System.out.println("calculLive: division par 0 dans le calcul de derivitive[i][0,1]");
        }
        if ( coreValsT[i] != 0.) {
          derivitive[i][2] = ((surfaceVals[i]/coreValsT[i]-1.0)-delta[i])/(tCdel);
        }
        else {
          System.out.println("calculLive: division par 0 dans le calcul de derivitive[i][2]");
        }
        if ( coreValsD[i] != 0.) {
          derivitive[i][3] = ((surfaceVals[i]/coreValsD[i]-1.0)-delta[i])/(dCdel);
        }
        else {
          System.out.println("calculLive: division par 0 dans le calcul de derivitive[i][3]");
        }
        derivitive[i][4] = delta[i];
      }

      System.out.println("maxDelta=" + maxDelta);
      bibli.correctionValeur(derivitive); // On corrige les conditions aux limites

    }
	
// on recupere les donnees depuis bibli

    tEffective = bibli.tEffective;
    dCentral = bibli.dCentral;
    tCentral = bibli.tCentral;
    luminosite = bibli.luminosite;

// je laisse les 5 lignes ci-dessous car "out" doit contenir toutes les grandeurs dfinies comme paramtres de sortie
    out.ajouterDouble("tEff", tEffective);
    out.ajouterDouble("dCentre", dCentral);
    out.ajouterDouble("tCentre", tCentral);
    out.ajouterDouble("luminosite", luminosite);
    out.ajouterDouble("masseJ", fitMassFraction);
    
// on lit les vecteurs de valeurs crs par dedans et dehors et on les met dans "out"

    for (int k=0 ; k < bibli.vecteurXout.size() ; k++ ) {
      out.ajouterDouble("m_out", ((Double)bibli.vecteurXout.elementAt(k)).doubleValue());
      out.ajouterDouble("t_out", ((Double)bibli.vecteurTout.elementAt(k)).doubleValue());
      out.ajouterDouble("d_out", ((Double)bibli.vecteurDout.elementAt(k)).doubleValue());
      out.ajouterDouble("r_out", ((Double)bibli.vecteurRout.elementAt(k)).doubleValue());
      out.ajouterDouble("l_out", ((Double)bibli.vecteurLout.elementAt(k)).doubleValue());
    }
    for (int k=0 ; k < bibli.vecteurXin.size() ; k++ ) {
      out.ajouterDouble("m_in", ((Double)bibli.vecteurXin.elementAt(k)).doubleValue());
      out.ajouterDouble("t_in", ((Double)bibli.vecteurTin.elementAt(k)).doubleValue());
      out.ajouterDouble("d_in", ((Double)bibli.vecteurDin.elementAt(k)).doubleValue());
      out.ajouterDouble("r_in", ((Double)bibli.vecteurRin.elementAt(k)).doubleValue());
      out.ajouterDouble("l_in", ((Double)bibli.vecteurLin.elementAt(k)).doubleValue());
    }

// on rinitialise les vecteurs pour l'itration suivante

    bibli.vecteurXout.removeAllElements();
    bibli.vecteurTout.removeAllElements();
    bibli.vecteurDout.removeAllElements();
    bibli.vecteurRout.removeAllElements();
    bibli.vecteurLout.removeAllElements();
    bibli.vecteurXin.removeAllElements();
    bibli.vecteurTin.removeAllElements();
    bibli.vecteurDin.removeAllElements();
    bibli.vecteurRin.removeAllElements();
    bibli.vecteurLin.removeAllElements();

// le calcul est-il termin ?
    if ( nIter >= maxiteration || maxDelta < bibli.modelPrecision ) {
// on
      finCalcul = true;
      nIter = 0;
      if ( maxDelta < bibli.modelPrecision ) {
        message = "Le modle a converg";
        System.out.println("Le modle a converg");
      }
      else {
        message = "Le modle n'a pas converg";
        System.out.println("Le modle n'a pas converg");
      }
      out.ajouterString("info", message);
      return(out);
    }
    else {
// off
      finIter = true;
      message = "Temp. eff. (K): "+bibli.arrondi(tEffective,2,false);
      message += "  Lum. (LSol): "+bibli.arrondi(luminosite,2,true);
      message += "  Temp. centr. (K): "+bibli.arrondi(tCentral,2,true);
      message += "  Dens. centr. (cgs): "+bibli.arrondi(dCentral,2,false);
      out.ajouterString("info", message);
      return(out);
    }
//        dormir(Math.round(entre2*1000)); // pour bloquer le programme pendant entre2 secondes
  }
    
    // calcul complet d'un coup (pas utilis dans cette appliquette)
  public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        return null;
  }

}

